/*
 *
 * CKSM doesn't work (not supported by server?)
 */
#include "globus_ftp_client.h"
#include "gridftp.h"

#ifdef GRIDFTP_IRODS_MODULE
#include "rsApiHandler.h"
#include "apiHeaderAll.h"
#endif

static globus_mutex_t lock;
static globus_cond_t cond;
static globus_bool_t done = GLOBUS_FALSE;
static globus_bool_t error = GLOBUS_TRUE;
char *cksm;

static void done_cb(
		void *user_arg,
		globus_ftp_client_handle_t *handle,
		globus_object_t *err) {

	if (err) {
		printErrorObject(err);
	}
	else {
		error = GLOBUS_FALSE;
		printInfo("gridftp cksm succeeded");
	}
	globus_mutex_lock(&lock);
	done = GLOBUS_TRUE;
	globus_cond_signal(&cond);
	globus_mutex_unlock(&lock);
}

int gridftp_cksm(char *url) {

	printInfo("gridftp cksm %s", url);

	globus_result_t result;
	globus_ftp_client_handle_t handle;
	globus_ftp_client_handleattr_t handle_attr;
	globus_ftp_client_operationattr_t operation_attr;
	globus_off_t offset = 0;
	globus_off_t length = -1;
	char *algorithm = "MD5";

	cksm = malloc(1024 * 1024 * sizeof(char));

	if (globus_module_activate(GLOBUS_FTP_CLIENT_MODULE) == GLOBUS_FAILURE) {
		printErrorMessage("globus_module_activate failed");
	}
	else {
		globus_mutex_init(&lock, GLOBUS_NULL);
		globus_cond_init(&cond, GLOBUS_NULL);

		result = globus_ftp_client_handleattr_init(&handle_attr);

		if (checkResult(result) != GLOBUS_SUCCESS) {
			printErrorMessage("globus_ftp_client_handleattr_init failed");
		}
		else {
			result = globus_ftp_client_operationattr_init(&operation_attr);

			if (checkResult(result) != GLOBUS_SUCCESS) {
				printErrorMessage("globus_ftp_client_operationattr_init failed");
			}
			else {
				result = globus_ftp_client_handle_init(&handle, &handle_attr);

				if (checkResult(result) != GLOBUS_SUCCESS) {
					printErrorMessage("globus_ftp_client_handle_init failed");
				}
				else {
					if (gridftp_feat_check(&handle, url, GLOBUS_FTP_CLIENT_FEATURE_CKSM) == GLOBUS_FTP_CLIENT_TRUE) {

						result = globus_ftp_client_cksm(
								&handle,
								url,
								&operation_attr,
								cksm,
								offset,
								length,
								algorithm,
								done_cb,
								0);

						if (checkResult(result) != GLOBUS_SUCCESS) {
							printErrorMessage("globus_ftp_client_cksm failed");
							done = GLOBUS_TRUE;
						}
					}
					else {
						printErrorMessage("cksm not supported by server");
						done = GLOBUS_TRUE;
					}

					globus_mutex_lock(&lock);

					while (!done) {
						globus_cond_wait(&cond, &lock);
					}
					globus_mutex_unlock(&lock);
				}
				result = globus_ftp_client_handle_destroy(&handle);
				checkResult(result);
			}
			result = globus_ftp_client_operationattr_destroy(&operation_attr);
			checkResult(result);
		}
		result = globus_ftp_client_handleattr_destroy(&handle_attr);
		checkResult(result);
	}
	globus_module_deactivate(GLOBUS_FTP_CLIENT_MODULE);

	if (error == GLOBUS_FALSE) {
#ifdef GRIDFTP_IRODS_MODULE
		printInfo("%s", cksm);
#else
		printf("%s\n", cksm);
#endif
	}
	return error;
}

#ifdef GRIDFTP_IRODS_MODULE

int msiGridftpCksm(msParam_t *url, msParam_t *outParam, ruleExecInfo_t *rei) {

	done = GLOBUS_FALSE;

	setUserProxy(rei->rsComm);

	if (gridftp_cksm((char *) url->inOutStruct) == GLOBUS_SUCCESS) {

		fillStrInMsParam(outParam, cksm);
		rei->status = 1;
	}
	else {
		rei->status = EXEC_CMD_ERROR;
	}
	return rei->status;
}

#endif
