#include "globus_ftp_client.h"
#include "gridftp.h"

#ifdef GRIDFTP_IRODS_MODULE
#include "rsApiHandler.h"
#include "apiHeaderAll.h"
#endif

static globus_mutex_t lock;
static globus_cond_t cond;
static globus_bool_t done = GLOBUS_FALSE;
static globus_bool_t error = GLOBUS_TRUE;

static void done_cb(
		void *user_arg,
		globus_ftp_client_handle_t *handle,
		globus_object_t *err) {

	if (err) {
		printErrorObject(err);
	}
	else {
		error = GLOBUS_FALSE;
		printInfo("gridftp cp succeeded");
	}
	globus_mutex_lock(&lock);
	done = GLOBUS_TRUE;
	globus_cond_signal(&cond);
	globus_mutex_unlock(&lock);
}

int gridftp_cp(char *src, char *dst) {

	printInfo("gridftp cp %s %s", src, dst);

	globus_result_t result;
	globus_ftp_client_handle_t handle;
	globus_ftp_client_handleattr_t handle_attr;
	globus_ftp_client_operationattr_t operation_attr;
	globus_ftp_control_parallelism_t parallelism;
	globus_ftp_control_tcpbuffer_t tcpbuffer;

	if (globus_module_activate(GLOBUS_FTP_CLIENT_MODULE) == GLOBUS_FAILURE) {
		printErrorMessage("globus_module_activate failed");
	}
	else {
		globus_mutex_init(&lock, GLOBUS_NULL);
		globus_cond_init(&cond, GLOBUS_NULL);

		result = globus_ftp_client_handleattr_init(&handle_attr);

		if (checkResult(result) != GLOBUS_SUCCESS) {
			printErrorMessage("globus_ftp_client_handleattr_init failed");
		}
		else {
			result = globus_ftp_client_handle_init(&handle, &handle_attr);

			if (checkResult(result) != GLOBUS_SUCCESS) {
				printErrorMessage("globus_ftp_client_handle_init failed");
			}
			else {
				result = globus_ftp_client_operationattr_init(&operation_attr);

				if (checkResult(result) != GLOBUS_SUCCESS) {
					printErrorMessage("globus_ftp_client_operationattr_init failed");
				}
				else {
					result = globus_ftp_client_operationattr_set_mode(&operation_attr, GLOBUS_FTP_CONTROL_MODE_EXTENDED_BLOCK);
					if (checkResult(result) != GLOBUS_SUCCESS) {
						printErrorMessage("globus_ftp_client_operationattr_set_mode failed");
					}

					/*	cache control channel connections for efficiency */
					result = globus_ftp_client_handleattr_set_cache_all(&handle_attr, (globus_bool_t) GLOBUS_TRUE);
					if (checkResult(result) != GLOBUS_SUCCESS) {
						printErrorMessage("globus_ftp_client_handleattr_set_cache_all failed");
					}

					/*	use 4 parallel data streams for high throughput */
					parallelism.mode = GLOBUS_FTP_CONTROL_PARALLELISM_FIXED;
					parallelism.fixed.size = 4;
					result = globus_ftp_client_operationattr_set_parallelism(&operation_attr, &parallelism);
					if (checkResult(result) != GLOBUS_SUCCESS) {
						printErrorMessage("globus_ftp_client_operationattr_set_parallelism failed");
					}

					/*	use large TCP windows for WANs with latency */
					tcpbuffer.mode =  GLOBUS_FTP_CONTROL_TCPBUFFER_FIXED;
					tcpbuffer.fixed.size = 1024 * 1024 * 1;
					result = globus_ftp_client_operationattr_set_tcp_buffer(&operation_attr, &tcpbuffer);
					if (checkResult(result) != GLOBUS_SUCCESS) {
						printErrorMessage("globus_ftp_client_operationattr_set_tcp_buffer failed");
					}

					result = globus_ftp_client_third_party_transfer(
							&handle,
							src,
							&operation_attr,
							dst,
							&operation_attr,
							GLOBUS_NULL,
							done_cb,
							0);

					if (checkResult(result) != GLOBUS_SUCCESS) {
						printErrorMessage("globus_ftp_client_third_party_transfer failed");
						done = GLOBUS_TRUE;
					}
					globus_mutex_lock(&lock);

					while (!done) {
						globus_cond_wait(&cond, &lock);
					}
					globus_mutex_unlock(&lock);
				}
				result = globus_ftp_client_operationattr_destroy(&operation_attr);
				checkResult(result);
			}
			result = globus_ftp_client_handle_destroy(&handle);
			checkResult(result);
		}
		result = globus_ftp_client_handleattr_destroy(&handle_attr);
		checkResult(result);
	}
	globus_module_deactivate(GLOBUS_FTP_CLIENT_MODULE);

	return error;
}

#ifdef GRIDFTP_IRODS_MODULE

int msiGridftpCp(msParam_t *src, msParam_t *dst, ruleExecInfo_t *rei) {

	done = GLOBUS_FALSE;

	setUserProxy(rei->rsComm);

	if (gridftp_cp((char *) src->inOutStruct, (char *) dst->inOutStruct) == GLOBUS_SUCCESS) {

		rei->status = 1;
	}
	else {
		rei->status = EXEC_CMD_ERROR;
	}
	return rei->status;
}

#endif
