#include "globus_ftp_client.h"
#include "gridftp.h"

#ifdef GRIDFTP_IRODS_MODULE
#include "rsApiHandler.h"
#include "apiHeaderAll.h"
#endif

static globus_mutex_t lock;
static globus_cond_t cond;
static globus_bool_t done = GLOBUS_FALSE;
static globus_bool_t error = GLOBUS_TRUE;
globus_bool_t exist = GLOBUS_FALSE;

static void done_cb(
		void *user_arg,
		globus_ftp_client_handle_t *handle,
		globus_object_t *err) {

	/* Error occurs when file does not exist, too. I don't know another
	 * handling than to ignore error at this point and assume, the file
	 * simply doesn't exist. At least, it isn't available at this time.
	 * Server errors are ignored here.
	 */
	if (err) {
		error = GLOBUS_FALSE;
//		printErrorObject(err);
#ifdef GRIDFTP_IRODS_MODULE
		printInfo("file does not exist");
#else
		printf("file does not exist\n");
#endif
	}
	else {
		error = GLOBUS_FALSE;
		exist = GLOBUS_TRUE;
#ifdef GRIDFTP_IRODS_MODULE
		printInfo("file exists");
#else
		printf("file exists\n");
#endif
	}
	globus_mutex_lock(&lock);
	done = GLOBUS_TRUE;
	globus_cond_signal(&cond);
	globus_mutex_unlock(&lock);
}

int gridftp_exist(char *url) {

	printInfo("gridftp exist %s", url);

	globus_result_t result;
	globus_ftp_client_handle_t handle;
	globus_ftp_client_handleattr_t handle_attr;
	globus_ftp_client_operationattr_t operation_attr;

	if (globus_module_activate(GLOBUS_FTP_CLIENT_MODULE) == GLOBUS_FAILURE) {
		printErrorMessage("globus_module_activate failed");
	}
	else {
		globus_mutex_init(&lock, GLOBUS_NULL);
		globus_cond_init(&cond, GLOBUS_NULL);

		result = globus_ftp_client_handleattr_init(&handle_attr);

		if (checkResult(result) != GLOBUS_SUCCESS) {
			printErrorMessage("globus_ftp_client_handleattr_init failed");
		}
		else  {
			result = globus_ftp_client_operationattr_init(&operation_attr);

			if (checkResult(result) != GLOBUS_SUCCESS) {
				printErrorMessage("globus_ftp_client_operationattr_init failed");
			}
			else {
				result = globus_ftp_client_handle_init(&handle, &handle_attr);

				if (checkResult(result) != GLOBUS_SUCCESS) {
					printErrorMessage("globus_ftp_client_handle_init failed");
				}
				else {
					result = globus_ftp_client_exists(
							&handle,
							url,
							&operation_attr,
							done_cb,
							0);

					if (checkResult(result) != GLOBUS_SUCCESS) {
						printErrorMessage("globus_ftp_client_exists failed");
						done = GLOBUS_TRUE;
					}
					globus_mutex_lock(&lock);

					while (!done) {
						globus_cond_wait(&cond, &lock);
					}
					globus_mutex_unlock(&lock);
				}
				result = globus_ftp_client_handle_destroy(&handle);
				checkResult(result);
			}
			result = globus_ftp_client_operationattr_destroy(&operation_attr);
			checkResult(result);
		}
		result = globus_ftp_client_handleattr_destroy(&handle_attr);
		checkResult(result);
	}
	globus_module_deactivate(GLOBUS_FTP_CLIENT_MODULE);

	return error;
}

#ifdef GRIDFTP_IRODS_MODULE

int msiGridftpExist(msParam_t *url, msParam_t *out, ruleExecInfo_t *rei) {

	done = GLOBUS_FALSE;

	setUserProxy(rei->rsComm);

	if (gridftp_exist((char *) url->inOutStruct) == GLOBUS_SUCCESS) {

		exist ? fillStrInMsParam(out, "YES"): fillStrInMsParam(out, "NO");
		rei->status = 1;
	}
	else {
		fillStrInMsParam(out, "NO");
		rei->status = EXEC_CMD_ERROR;
	}
	return rei->status;
}

#endif
