/*
 * TODO: check for checking right feature names
 */
#include "globus_ftp_client.h"
#include "gridftp.h"

#ifdef GRIDFTP_IRODS_MODULE
#include "rsApiHandler.h"
#include "apiHeaderAll.h"
#endif

#define ARRAY_SIZE 19

static globus_mutex_t lock;
static globus_cond_t cond;
static globus_bool_t done = GLOBUS_FALSE;
static globus_bool_t error = GLOBUS_TRUE;

char *feature_names[ARRAY_SIZE] = {
	    "GLOBUS_FTP_CLIENT_FEATURE_RETRBUFSIZE",
	    "GLOBUS_FTP_CLIENT_FEATURE_RBUFSZ",
	    "GLOBUS_FTP_CLIENT_FEATURE_RBUFSIZ",
	    "GLOBUS_FTP_CLIENT_FEATURE_STORBUFSIZE",
	    "GLOBUS_FTP_CLIENT_FEATURE_SBUSSZ",
	    "GLOBUS_FTP_CLIENT_FEATURE_SBUFSIZ",
	    "GLOBUS_FTP_CLIENT_FEATURE_BUFSIZE",
	    "GLOBUS_FTP_CLIENT_FEATURE_SBUF",
	    "GLOBUS_FTP_CLIENT_FEATURE_ABUF",

	    "GLOBUS_FTP_CLIENT_FEATURE_REST_STREAM",
	    "GLOBUS_FTP_CLIENT_FEATURE_PARALLELISM",
	    "GLOBUS_FTP_CLIENT_FEATURE_DCAU",
	    "GLOBUS_FTP_CLIENT_FEATURE_ESTO",
	    "GLOBUS_FTP_CLIENT_FEATURE_ERET",
	    "GLOBUS_FTP_CLIENT_FEATURE_SIZE",
	    "GLOBUS_FTP_CLIENT_FEATURE_CKSM",
	    "GLOBUS_FTP_CLIENT_FEATURE_MLST",
	    "GLOBUS_FTP_CLIENT_FEATURE_CHMOD",
	    "GLOBUS_FTP_CLIENT_FEATURE_MAX"};
globus_ftp_client_features_t features;
globus_ftp_client_tristate_t feature_supported[ARRAY_SIZE];
globus_bool_t internal_call = GLOBUS_FALSE;
globus_ftp_client_handle_t *handle;

static void done_cb(
		void *user_arg,
		globus_ftp_client_handle_t *_handle,
		globus_object_t *err) {

	if (err) {
		printErrorObject(err);
	}
	else {
		error = GLOBUS_FALSE;
		printInfo("gridftp feat succeeded");
	}
	globus_mutex_lock(&lock);
	done = GLOBUS_TRUE;
	globus_cond_signal(&cond);
	globus_mutex_unlock(&lock);
}

int gridftp_feat(char *url) {

	done = GLOBUS_FALSE;

	printInfo("gridftp feat %s", url);
	printErrorMessage("ATTENTION: gridftp feat isn't fully supported. Values may be wrong.");

	globus_result_t result;
	globus_ftp_client_handleattr_t handle_attr;
	globus_ftp_client_tristate_t answer;

	if (internal_call == GLOBUS_FALSE) {

		handle = malloc(sizeof(globus_ftp_client_handle_t));

		if (globus_module_activate(GLOBUS_FTP_CLIENT_MODULE) == GLOBUS_FAILURE) {
			printErrorMessage("globus_module_activate failed");
		}
		globus_mutex_init(&lock, NULL);
		globus_cond_init(&cond, NULL);

		result = globus_ftp_client_handleattr_init(&handle_attr);
		checkResult(result);
		result = globus_ftp_client_handle_init(handle, &handle_attr);
		checkResult(result);
	}
	result = globus_ftp_client_features_init(&features);
	checkResult(result);

	result = globus_ftp_client_feat(
			handle,
			url,
			NULL,
			&features,
			done_cb,
			NULL);

	if (checkResult(result) != GLOBUS_SUCCESS) {
		done = GLOBUS_TRUE;
	}
	globus_mutex_lock(&lock);

	while (!done) {
		globus_cond_wait(&cond, &lock);
	}
	globus_mutex_unlock(&lock);

	if (internal_call == GLOBUS_FALSE) {

		result = globus_ftp_client_handle_destroy(handle);
		checkResult(result);
		result = globus_ftp_client_handleattr_destroy(&handle_attr);
		checkResult(result);
	}

	if (internal_call == GLOBUS_FALSE) {

		printInfo("Features supported by server:");
		printInfo("ATTENTION: may contain wrong values!\n");
	}

	int i;
//	for (i = 0; i < GLOBUS_FTP_CLIENT_FEATURE_MAX; i++) {
	for (i = 0; i < ARRAY_SIZE; i++) {

		char *line = malloc(265 * sizeof(char));
		result = globus_ftp_client_is_feature_supported(&features, &answer, i);
		sprintf(line, "%d\t%s\t", i, feature_names[i]);

		feature_supported[i] = answer;

		if (answer == GLOBUS_FTP_CLIENT_TRUE)
			strcat(line, "yes");
		else if (answer == GLOBUS_FTP_CLIENT_FALSE)
			strcat(line, "no");
		else if (answer == GLOBUS_FTP_CLIENT_MAYBE)
			strcat(line, "maybe");
		else {
			printErrorMessage("feature %d has unsupported value %d", i, answer);
			exit(-1);
		}
		if (internal_call == GLOBUS_FALSE) printInfo(line);
	}
	result = globus_ftp_client_features_destroy(&features);
	checkResult(result);

	if (internal_call == GLOBUS_FALSE) {
		globus_module_deactivate(GLOBUS_FTP_CLIENT_MODULE);
	}

	return error;
}

/*
 * Internal call to check for a single feature supported by remote server.
 */
globus_ftp_client_tristate_t gridftp_feat_check(globus_ftp_client_handle_t *handle_param, char *url, globus_ftp_client_probed_feature_t feature) {

#ifdef GRIDFTP_DISABLE_FEATURE_CHECK
	printInfo("gridftp_feat_check disabled");
	return GLOBUS_FTP_CLIENT_TRUE;
#else

	printInfo("feature check for %s (%i)", feature_names[feature], feature);
	internal_call = GLOBUS_TRUE;
	handle = handle_param;

	if (gridftp_feat(url) == GLOBUS_SUCCESS) {

		if (feature < GLOBUS_FTP_CLIENT_FEATURE_MAX && feature >= 0) {

			char *tmp;

			if (feature_supported[feature] == GLOBUS_FTP_CLIENT_TRUE)
				tmp = "yes";
			else if (feature_supported[feature] == GLOBUS_FTP_CLIENT_FALSE)
				tmp = "no";
			else if (feature_supported[feature] == GLOBUS_FTP_CLIENT_MAYBE)
				tmp = "maybe";
			else {
				printErrorMessage("feature %s has unsupported value %d", feature_names[feature], feature_supported[feature]);
				return GLOBUS_FTP_CLIENT_FALSE;
			}
			printInfo("%s supported: %s", feature_names[feature], tmp);
			return feature_supported[feature];
		}
		printErrorMessage("no feature %i", feature);
	}
	return GLOBUS_FTP_CLIENT_FALSE;
#endif
}
