#include "globus_ftp_client.h"
#include "gridftp.h"

#ifdef GRIDFTP_IRODS_MODULE
#include "rsApiHandler.h"
#include "apiHeaderAll.h"
#include "dataObjGet.h"
#include "dataObjOpen.h"
#include "dataObjClose.h"
#include "rodsClient.h"
ruleExecInfo_t *rei;

#endif	//#else
#include <sys/stat.h>
//#endif

#define BUFFER_SIZE 2048
static globus_mutex_t lock;
static globus_cond_t cond;
static globus_bool_t done = GLOBUS_FALSE;
static globus_bool_t error = GLOBUS_TRUE;

static void done_cb(
		void *user_arg,
		globus_ftp_client_handle_t *handle,
		globus_object_t *err) {

	if (err) {
		printErrorObject(err);
	}
	else {
		printInfo("gridftp get succeeded");
		error = GLOBUS_FALSE;
	}
	globus_mutex_lock(&lock);
	done = GLOBUS_TRUE;
	globus_cond_signal(&cond);
	globus_mutex_unlock(&lock);
}

#ifdef GRIDFTP_IRODS_MODULE

static void module_data_cb(
		void *user_arg,
		globus_ftp_client_handle_t *handle,
		globus_object_t *err,
		globus_byte_t *buffer,
		globus_size_t length,
		globus_off_t offset,
		globus_bool_t eof) {

	if (err) {
		printErrorObject(err);
	}
	else {

		int status;
		bytesBuf_t data;
		dataObjWriteInp_t writeParam;

		memset(&writeParam, 0, sizeof(dataObjWriteInp_t));
		writeParam.l1descInx = *(int *) user_arg;
		writeParam.len = length;

		data.len = length;
		data.buf = (void *) buffer;

		status = rsDataObjWrite(
				rei->rsComm,
				&writeParam,
				&data);

		if (status < 0) {
			printInfo("rsDataObjWrite failed");
		}

		if (!eof) {
			globus_result_t result = globus_ftp_client_register_read(
					handle,
					buffer,
					BUFFER_SIZE,
					module_data_cb,
					user_arg);

			if (checkResult(result) != GLOBUS_SUCCESS) {
				printErrorMessage("globus_ftp_client_register_read failed");
			}
		}
	}
}

//	GRIDFTP_IRODS_MODULE
int gridftp_module_get(char *src, char *dst) {

	globus_result_t result;
	globus_ftp_client_handle_t handle;
	globus_ftp_client_handleattr_t handle_attr;
	globus_ftp_client_operationattr_t operation_attr;
	globus_byte_t buffer[BUFFER_SIZE];

	if ('/' == dst[strlen(dst) - 1]) {
		dst[strlen(dst) -1] = '\0';
	}

    int status1;
	rodsLong_t* id;
	status1 = isColl(rei->rsComm, dst, id);

	if (0 == status1) { // is path
		strncat(dst, "/", 1);
		char eins[512], zwei[512];
		strncpy(eins, src, strlen(src) + 1);
		strncpy(zwei, dst, strlen(dst) + 1);

		char *tok;
		char *tmptok = strtok(eins, "/");

		while (tmptok != NULL) {
			tok = tmptok;
			tmptok = strtok(NULL, "/");
		}
		char drei[512];
		strncpy(drei, tok, strlen(tok) + 1);

		if (zwei[strlen(zwei) - 1] != '/') {
			char vier[512] = "/";
			strncpy(drei, tok, strlen(tok) + 1);
			strncat(vier, drei, strlen(drei));
			strncat(zwei, vier, strlen(vier));
		}
		else {
			strncat(zwei, drei, strlen(drei));
		}
		dst = zwei;
	}

	printInfo("gridftp module get %s %s", src, dst);

	if (globus_module_activate(GLOBUS_FTP_CLIENT_MODULE) == GLOBUS_FAILURE) {
		printErrorMessage("globus_module_activate failed");
	}
	else {
		globus_mutex_init(&lock, GLOBUS_NULL);
		globus_cond_init(&cond, GLOBUS_NULL);

		result = globus_ftp_client_handleattr_init(&handle_attr);

		if (checkResult(result) != GLOBUS_SUCCESS) {
			printErrorMessage("globus_ftp_client_handleattr_init failed");
		}
		else {
			result = globus_ftp_client_operationattr_init(&operation_attr);

			if (checkResult(result) != GLOBUS_SUCCESS) {
				printErrorMessage("globus_ftp_client_operationattr_init failed");
			}
			else {
				result = globus_ftp_client_handle_init(&handle, &handle_attr);

				if (checkResult(result) != GLOBUS_SUCCESS) {
					printErrorMessage("globus_ftp_client_handle_init failed");
				}
				else {
					int fd;

					result = globus_ftp_client_get(
							&handle,
							src,
							&operation_attr,
							GLOBUS_NULL,
							done_cb,
							NULL);

					if (checkResult(result) != GLOBUS_SUCCESS) {
						printErrorMessage("globus_ftp_client_get failed");
						done = GLOBUS_TRUE;
					}
					else {
						dataObjInp_t createParam;
						memset(&createParam, 0, sizeof(dataObjInp_t));
						rstrcpy(createParam.objPath, dst, MAX_NAME_LEN);
						createParam.openFlags = O_WRONLY;
					//	createParam.createMode = 664;
						fd = rsDataObjCreate(rei->rsComm, &createParam);

						if (fd > 0) {
							result = globus_ftp_client_register_read(
									&handle,
									buffer,
									BUFFER_SIZE,
									module_data_cb,
									(void *) &fd);

							if (checkResult(result) != GLOBUS_SUCCESS) {
								printErrorMessage("globus_ftp_client_register_read failed");
								done = GLOBUS_TRUE;
							}
						}
						else {
							printInfo("rsDataObjCreate failed, fd = %i", fd);
							done = GLOBUS_TRUE;
						}
					}
					globus_mutex_lock(&lock);

					while (!done) {
						globus_cond_wait(&cond, &lock);
					}
					globus_mutex_unlock(&lock);

					if (fd > 0) {
						dataObjCloseInp_t closeParam;
						memset(&closeParam, 0, sizeof(dataObjCloseInp_t));
						closeParam.l1descInx = fd;
						rsDataObjClose(rei->rsComm, &closeParam);
					}
				}
				result = globus_ftp_client_handle_destroy(&handle);
				checkResult(result);
			}
			result = globus_ftp_client_operationattr_destroy(&operation_attr);
			checkResult(result);
		}
		result = globus_ftp_client_handleattr_destroy(&handle_attr);
		checkResult(result);
	}
	globus_module_deactivate(GLOBUS_FTP_CLIENT_MODULE);

	return error;
}

int msiGridftpGet(msParam_t *srcParam, msParam_t *dstParam, ruleExecInfo_t *p_rei) {

	done = GLOBUS_FALSE;

	rei = p_rei;
	char *src = (char*) malloc(strlen(srcParam->inOutStruct) * sizeof(char));
	char *dst = (char*) malloc(strlen(dstParam->inOutStruct) * sizeof(char));
	strcpy(src, srcParam->inOutStruct);
	strcpy(dst, dstParam->inOutStruct);

	setUserProxy(rei->rsComm);

	if (gridftp_module_get(src, dst) == GLOBUS_SUCCESS) {

		rei->status = 1; //ok
	}
	else {
		rei->status = EXEC_CMD_ERROR; //error
	}
	return rei->status;
}

#endif	//#else	// not GRIDFTP_IRODS_MODULE

static void data_cb(
		void *user_arg,
		globus_ftp_client_handle_t *handle,
		globus_object_t *err,
		globus_byte_t *buffer_t,
		globus_size_t length,
		globus_off_t offset,
		globus_bool_t eof) {

	if (err) {
		printErrorObject(err);
	}
	else {
		fwrite(buffer_t, 1, length, (FILE *) user_arg);

		if (!eof) {
			globus_result_t result = globus_ftp_client_register_read(
					handle,
					buffer_t,
					BUFFER_SIZE,
					data_cb,
					user_arg);
			if (checkResult(result) != GLOBUS_SUCCESS) {
				printErrorMessage("globus_ftp_client_register_read failed");
			}
		}
	}
}

int gridftp_get(char *src, char *dst) {

	printInfo("gridftp get %s %s", src, dst);

	globus_result_t result;
	globus_ftp_client_handle_t handle;
	globus_ftp_client_handleattr_t handle_attr;
	globus_ftp_client_operationattr_t operation_attr;
	globus_byte_t buffer[BUFFER_SIZE];
	globus_size_t buffer_length = BUFFER_SIZE;
	struct stat attribute;

	// check whether dst is a directory and create correct filename
	if (stat(dst, &attribute) != -1) {
		char eins[100], zwei[100];
		strncpy(eins, src, strlen(src) + 1);
		strncpy(zwei, dst, strlen(dst) + 1);

		if (attribute.st_mode & S_IFDIR) {	// is a directory
			char *tok;
			char *tmptok = strtok(eins, "/");

			while (tmptok != NULL) {
				tok = tmptok;
				tmptok = strtok(NULL, "/");
			}
			char drei[100];
			strncpy(drei, tok, strlen(tok) + 1);

			if (zwei[strlen(zwei) - 1] != '/') {
				char vier[100] = "/";
				strncpy(drei, tok, strlen(tok) + 1);
				strncat(vier, drei, strlen(drei));
				strncat(zwei, vier, strlen(vier));
			}
			else {
				strncat(zwei, drei, strlen(drei));
			}
			dst = zwei;
		}
	}

	if (globus_module_activate(GLOBUS_FTP_CLIENT_MODULE) == GLOBUS_FAILURE) {
		printErrorMessage("globus_module_activate failed");
	}
	else {
		globus_mutex_init(&lock, GLOBUS_NULL);
		globus_cond_init(&cond, GLOBUS_NULL);

		result = globus_ftp_client_handleattr_init(&handle_attr);

		if (checkResult(result) != GLOBUS_SUCCESS) {
			printErrorMessage("globus_ftp_client_handleattr_init failed");
		}
		else {
			result = globus_ftp_client_operationattr_init(&operation_attr);

			if (checkResult(result) != GLOBUS_SUCCESS) {
				printErrorMessage("globus_ftp_client_operationattr_init failed");
			}
			else {
				result = globus_ftp_client_handle_init(&handle, &handle_attr);

				if (checkResult(result) != GLOBUS_SUCCESS) {
					printErrorMessage("globus_ftp_client_handle_init failed");
				}
				else {
					FILE *fp;

					result = globus_ftp_client_get(
							&handle,
							src,
							&operation_attr,
							GLOBUS_NULL,
							done_cb,
							NULL);

					if (checkResult(result) != GLOBUS_SUCCESS) {
						printErrorMessage("globus_ftp_client_get failed");
						done = GLOBUS_TRUE;
					}
					else {
						fp = fopen(dst, "a");

						globus_ftp_client_register_read(
								&handle,
								buffer,
								buffer_length,
								data_cb,
								(void *) fp);

						if (checkResult(result) != GLOBUS_SUCCESS) {
							printErrorMessage("globus_ftp_client_register_read failed");
							done = GLOBUS_TRUE;
						}
					}
					globus_mutex_lock(&lock);

					while (!done) {
						globus_cond_wait(&cond, &lock);
					}
					globus_mutex_unlock(&lock);

					fclose(fp);
				}
				result = globus_ftp_client_handle_destroy(&handle);
				checkResult(result);
			}
			result = globus_ftp_client_operationattr_destroy(&operation_attr);
			checkResult(result);
		}
		result = globus_ftp_client_handleattr_destroy(&handle_attr);
		checkResult(result);
	}
	globus_module_deactivate(GLOBUS_FTP_CLIENT_MODULE);

	return error;
}

//#endif	//GRIDFTP_IRODS_MODULE
