#include "globus_ftp_client.h"
#include "gridftp.h"

#ifdef GRIDFTP_IRODS_MODULE
#include "objMetaOpr.h"
#include "rsApiHandler.h"
#include "apiHeaderAll.h"
char *listing;
#endif

static globus_mutex_t lock;
static globus_cond_t cond;
static globus_bool_t done = GLOBUS_FALSE;
static globus_bool_t error = GLOBUS_TRUE;
#define SIZE 2048

static void done_cb(
		void *user_arg,
		globus_ftp_client_handle_t *handle,
		globus_object_t *err) {

	if (err) {
		printErrorObject(err);
	}
	else {
		error = GLOBUS_FALSE;
		printInfo("gridftp ls succeeded");
	}
	globus_mutex_lock(&lock);
	done = GLOBUS_TRUE;
	globus_cond_signal(&cond);
	globus_mutex_unlock(&lock);
}

static void data_cb(
        void *user_arg,
        globus_ftp_client_handle_t *handle,
        globus_object_t *err,
        globus_byte_t *buffer,
        globus_size_t length,
        globus_off_t offset,
        globus_bool_t eof) {


#ifdef GRIDFTP_IRODS_MODULE
	strncat(listing, buffer, length);
#else
	fwrite(buffer, 1, length, stdout);
#endif

	if (!eof) {
		globus_result_t result = globus_ftp_client_register_read(
				handle,
				buffer,
				SIZE,
				data_cb,
				NULL);
		if (checkResult(result) != GLOBUS_SUCCESS) {
			printErrorMessage("globus_ftp_client_register_read failed");
		}
	}
}

int gridftp_ls(char *url) {

	printInfo("gridftp ls %s", url);

#ifdef GRIDFTP_IRODS_MODULE
	listing = (char *) malloc(5120 * sizeof(char));
	memset(listing, 0, 5120);
#endif

	globus_result_t result;
	globus_ftp_client_handle_t handle;
	globus_ftp_client_handleattr_t handle_attr;
	globus_ftp_client_operationattr_t operation_attr;
	globus_byte_t buffer[SIZE];
	globus_size_t buffer_length = SIZE;

	if (globus_module_activate(GLOBUS_FTP_CLIENT_MODULE) == GLOBUS_FAILURE) {
		printErrorMessage("globus_module_activate failed");
	}
	else {
		globus_mutex_init(&lock, GLOBUS_NULL);
		globus_cond_init(&cond, GLOBUS_NULL);

		result = globus_ftp_client_handleattr_init(&handle_attr);

		if (checkResult(result) != GLOBUS_SUCCESS) {
			printErrorMessage("globus_ftp_client_handleattr_init failed");
		}
		else  {
			result = globus_ftp_client_operationattr_init(&operation_attr);

			if (checkResult(result) != GLOBUS_SUCCESS) {
				printErrorMessage("globus_ftp_client_operationattr_init failed");
			}
			else {
				result = globus_ftp_client_operationattr_set_type(&operation_attr, GLOBUS_FTP_CONTROL_TYPE_ASCII);
				if (checkResult(result) != GLOBUS_SUCCESS) {
					printErrorMessage("globus_ftp_client_operationattr_set_type failed");
				}
				else {
					result = globus_ftp_client_handle_init(&handle, &handle_attr);

					if (checkResult(result) != GLOBUS_SUCCESS) {
						printErrorMessage("globus_ftp_client_handle_init failed");
					}
					else  {
						result = globus_ftp_client_list(
								&handle,
								url,
								&operation_attr,
								done_cb,
								0);
						if (checkResult(result) != GLOBUS_SUCCESS) {
							printErrorMessage("globus_ftp_client_list failed");
						}
						else  {
							result = globus_ftp_client_register_read(
									&handle,
									buffer,
									buffer_length,
									data_cb,
									NULL);
							if (checkResult(result) != GLOBUS_SUCCESS) {
								printErrorMessage("globus_ftp_client_register_read failed");
								done = GLOBUS_TRUE;
							}
							globus_mutex_lock(&lock);

							while (!done) {
								globus_cond_wait(&cond, &lock);
							}
							globus_mutex_unlock(&lock);
						}
					}
					result = globus_ftp_client_handle_destroy(&handle);
					checkResult(result);
				}
			}
			result = globus_ftp_client_operationattr_destroy(&operation_attr);
			checkResult(result);
		}
		result = globus_ftp_client_handleattr_destroy(&handle_attr);
		checkResult(result);
	}
	globus_module_deactivate(GLOBUS_FTP_CLIENT_MODULE);

#ifdef GRIDFTP_IRODS_MODULE

	char *url_ = url;

	if (*(url + strlen(url) - 1) != '/') {

		url_ = (char *) malloc((strlen(url) + 1) * sizeof(char));
		strncpy(url_, url, strlen(url));
		strncat(url_, "/", 1);
	}

	int url_len = strlen(url_);
	int i, token_cnt = 0;

	for (i = 0; i < strlen(listing); i++) {

		if (*(listing + i) == '\n') token_cnt++;
	}

	char *files = (char *) malloc(((url_len * token_cnt) + strlen(listing)) * sizeof(char));
	memset(files, 0, url_len * token_cnt + strlen(listing));

	char *token = strtok(listing, "\n");

	while (token != NULL) {
		strncat(files, url_, url_len);
		strncat(files, token, strlen(token));
		strncat(files, ",", 1);
		token = strtok(NULL, "\n");
	}

	*(files + strlen(files) - 1) = '\0';
	printInfo("%i files: %s", token_cnt, files);

	free(listing);
	listing = files;
#endif

	return error;
}

#ifdef GRIDFTP_IRODS_MODULE

int msiGridftpLs(msParam_t *url, msParam_t *outParam, ruleExecInfo_t *rei) {

	done = GLOBUS_FALSE;

	setUserProxy(rei->rsComm);

	if (gridftp_ls((char *) url->inOutStruct) == GLOBUS_SUCCESS) {

		fillStrInMsParam(outParam, listing);
		rei->status = 1;
	}
	else {
		rei->status = EXEC_CMD_ERROR;
	}
	return rei->status;
}

#endif
