#include "globus_ftp_client.h"
#include "gridftp.h"

#ifdef GRIDFTP_IRODS_MODULE
#include "rsApiHandler.h"
#include "apiHeaderAll.h"
#include "dataObjOpen.h"
#include "dataObjClose.h"
#include "dataObjRead.h"

#include "rods.h"
#include "rcMisc.h"
#include "procApiRequest.h"
#include "apiNumber.h"
#include "initServer.h"
#include "dataObjInpOut.h"
#include "fileOpen.h"
#include "reGlobalsExtern.h"
#include "rsGlobalExtern.h"
#include "rcGlobalExtern.h"
#include "rcConnect.h"

ruleExecInfo_t *rei;
#endif

static globus_mutex_t lock;
static globus_cond_t cond;
static globus_bool_t done = GLOBUS_FALSE;
static globus_bool_t error = GLOBUS_TRUE;
#define BUFFER_SIZE 2048

static void done_cb(
		void *user_arg,
		globus_ftp_client_handle_t *handle,
		globus_object_t *err) {

	if (err) {
		printErrorObject(err);
	}
	else {
		printInfo("gridftp put succeeded");
		error = GLOBUS_FALSE;
	}
	globus_mutex_lock(&lock);
	done = GLOBUS_TRUE;
	globus_cond_signal(&cond);
	globus_mutex_unlock(&lock);
}

#ifdef GRIDFTP_IRODS_MODULE

static void module_data_cb(
		void *user_arg,
		globus_ftp_client_handle_t *handle,
		globus_object_t *err,
		globus_byte_t *buffer,
		globus_size_t length,
		globus_off_t offset,
		globus_bool_t eof) {

	if (err) {
		printErrorObject(err);
	}
	else if (!eof) {

		int status;
		bytesBuf_t data;
		dataObjReadInp_t readParam;

		memset(&data, 0, sizeof(bytesBuf_t));
		memset(&readParam, 0, sizeof(dataObjReadInp_t));

		readParam.l1descInx = *(int *) user_arg;
		readParam.len = BUFFER_SIZE;

		data.len = BUFFER_SIZE;
		data.buf = (void *) malloc(BUFFER_SIZE);

		status = rsDataObjRead(
				rei->rsComm,
				&readParam,
				&data);

		if (status > 2) {

			globus_byte_t *buffer = (globus_byte_t *) malloc(data.len * sizeof(globus_byte_t));//(globus_byte_t *) data.buf;//moveBytes(&data);
			memset(buffer, 0, data.len * sizeof(globus_byte_t));
			memcpy(buffer, data.buf, data.len);

			eof = (data.len == readParam.len) ? GLOBUS_FALSE : GLOBUS_TRUE;

			globus_result_t result = globus_ftp_client_register_write(
					handle,
					buffer,
					data.len,
					offset + data.len,
					eof,
					module_data_cb,
					(void *) user_arg);

			if (checkResult(result) != GLOBUS_SUCCESS) {
				printErrorMessage("globus_ftp_client_register_write failed");
			}
		}
		else {
			printInfo("rsDataObjRead failed");
		}
	}
}

//	GRIDFTP_IRODS_MODULE
int gridftp_module_put(char *src, char *dst) {

	printInfo("gridftp module put %s %s", src, dst);

	globus_result_t result;
	globus_ftp_client_handle_t handle;
	globus_ftp_client_handleattr_t handle_attr;
	globus_ftp_client_operationattr_t operation_attr;

	if (globus_module_activate(GLOBUS_FTP_CLIENT_MODULE) == GLOBUS_FAILURE) {
		printErrorMessage("globus_module_activate failed");
	}
	else {
		globus_mutex_init(&lock, GLOBUS_NULL);
		globus_cond_init(&cond, GLOBUS_NULL);

		result = globus_ftp_client_handleattr_init(&handle_attr);

		if (checkResult(result) != GLOBUS_SUCCESS) {
			printErrorMessage("globus_ftp_client_handleattr_init failed");
		}
		else {
			result = globus_ftp_client_operationattr_init(&operation_attr);

			if (checkResult(result) != GLOBUS_SUCCESS) {
				printErrorMessage("globus_ftp_client_operationattr_init failed");
			}
			else {
				result = globus_ftp_client_handle_init(&handle, &handle_attr);

				if (checkResult(result) != GLOBUS_SUCCESS) {
					printErrorMessage("globus_ftp_client_handle_init failed");
				}
				else {
					result = globus_ftp_client_put(
							&handle,
							dst,
							&operation_attr,
							GLOBUS_NULL,
							done_cb,
							0);

					if (checkResult(result) != GLOBUS_SUCCESS) {
						printErrorMessage("globus_ftp_client_put failed");
						done = GLOBUS_TRUE;
					}
					else {
						int fd;
						dataObjInp_t openParam;
						memset(&openParam, 0, sizeof(dataObjInp_t));
						rstrcpy(openParam.objPath, src, MAX_NAME_LEN);
						openParam.openFlags = O_RDONLY;

						fd = rsDataObjOpen(rei->rsComm, &openParam);

						if (fd > 0) {
							result = globus_ftp_client_register_write(
									&handle,
									(globus_byte_t *) "",
									0,
									0,
									GLOBUS_FALSE,
									module_data_cb,
									(void *) &fd);

							if (checkResult(result) != GLOBUS_SUCCESS) {
								printErrorMessage("globus_ftp_client_register_write failed");
								done = GLOBUS_TRUE;
							}
						}
						else {
							printErrorMessage("rsDataObjOpen failed, fd = %i", fd);
							done = GLOBUS_TRUE;
						}
						globus_mutex_lock(&lock);

						while (!done) {
							globus_cond_wait(&cond, &lock);
						}
						globus_mutex_unlock(&lock);

						if (0 > fd) {
							dataObjCloseInp_t closeParam;
							memset(&closeParam, 0, sizeof(dataObjCloseInp_t));
							closeParam.l1descInx = fd;
							rsDataObjClose(rei->rsComm, &closeParam);
						}
					}
				}
				result = globus_ftp_client_handle_destroy(&handle);
				checkResult(result);
			}
			result = globus_ftp_client_operationattr_destroy(&operation_attr);
			checkResult(result);
		}
		result = globus_ftp_client_handleattr_destroy(&handle_attr);
		checkResult(result);
	}
	globus_module_deactivate(GLOBUS_FTP_CLIENT_MODULE);

	return error;
}

int msiGridftpPut(msParam_t *src, msParam_t *dst, ruleExecInfo_t *rei_p) {

	done = GLOBUS_FALSE;

	rei = rei_p;

	setUserProxy(rei->rsComm);

	if (gridftp_module_put((char *) src->inOutStruct, (char *) dst->inOutStruct) == GLOBUS_SUCCESS) {

		rei->status = 1; //ok
	}
	else {
		rei->status = EXEC_CMD_ERROR; //error
	}
	return rei->status;
}

#endif //#else	//ifndef GRIDFTP_IRODS_MODULE

static void data_cb(
		void *user_arg,
		globus_ftp_client_handle_t *handle,
		globus_object_t *err,
		globus_byte_t *buffer,
		globus_size_t length,
		globus_off_t offset,
		globus_bool_t eof) {

	if (err) {
		printErrorObject(err);
	}
	else if (!eof) {
		globus_result_t result;
		FILE *fd = (FILE *) user_arg;
		int rc;
		rc = fread(buffer, 1, BUFFER_SIZE, fd);

		if (ferror(fd) != 0) {
			printErrorMessage("errno %d while read file", errno);
		}
		result = globus_ftp_client_register_write(
				handle,
				buffer,
				rc,
				offset + length,
				feof(fd) != 0,
				data_cb,
				(void *) fd);

		if (checkResult(result) != GLOBUS_SUCCESS) {
			printErrorMessage("globus_ftp_client_register_write failed");
		}
	}
}

int gridftp_put(char *src, char *dst) {

	printInfo("gridftp put %s %s", src, dst);

	globus_result_t result;
	globus_ftp_client_handle_t handle;
	globus_ftp_client_handleattr_t handle_attr;
	globus_ftp_client_operationattr_t operation_attr;
	globus_byte_t buffer[BUFFER_SIZE];

	FILE *file;
	file = fopen(src, "r");

	if (NULL == file) {
		printErrorMessage("can't open file: %s", src);
		return 1;
	}

	if (globus_module_activate(GLOBUS_FTP_CLIENT_MODULE) == GLOBUS_FAILURE) {
		printErrorMessage("globus_module_activate failed");
	}
	else {
		globus_mutex_init(&lock, GLOBUS_NULL);
		globus_cond_init(&cond, GLOBUS_NULL);

		result = globus_ftp_client_handleattr_init(&handle_attr);

		if (checkResult(result) != GLOBUS_SUCCESS) {
			printErrorMessage("globus_ftp_client_handleattr_init failed");
		}
		else {
			result = globus_ftp_client_operationattr_init(&operation_attr);

			if (checkResult(result) != GLOBUS_SUCCESS) {
				printErrorMessage("globus_ftp_client_operationattr_init failed");
			}
			else {
				result = globus_ftp_client_handle_init(&handle, &handle_attr);

				if (checkResult(result) != GLOBUS_SUCCESS) {
					printErrorMessage("globus_ftp_client_handle_init failed");
				}
				else {
					result = globus_ftp_client_put(
							&handle,
							dst,
							&operation_attr,
							GLOBUS_NULL,
							done_cb,
							0);

					if (checkResult(result) != GLOBUS_SUCCESS) {
						printErrorMessage("globus_ftp_client_put failed");
						done = GLOBUS_TRUE;
					}
					else {
						int rc;
						rc = fread(buffer, 1, BUFFER_SIZE, file);
						result = globus_ftp_client_register_write(
								&handle,
								buffer,
								rc,
								0,
								feof(file) != 0,
								data_cb,
								(void *) file);

						if (checkResult(result) != GLOBUS_SUCCESS) {
							printErrorMessage("globus_ftp_client_register_write failed");
							done = GLOBUS_TRUE;
						}
					}
					globus_mutex_lock(&lock);

					while (!done) {
						globus_cond_wait(&cond, &lock);
					}
					globus_mutex_unlock(&lock);
				}
				result = globus_ftp_client_handle_destroy(&handle);
				checkResult(result);
			}
			result = globus_ftp_client_operationattr_destroy(&operation_attr);
			checkResult(result);
		}
		result = globus_ftp_client_handleattr_destroy(&handle_attr);
		checkResult(result);
	}
	globus_module_deactivate(GLOBUS_FTP_CLIENT_MODULE);

	fclose(file);
	return error;
}
//#endif	//GRIDFTP_IRODS_MODULE
