/*
 *	Helpers for gridftp_client
 */

#include "gridftp.h"
#include "globus_ftp_client.h"

#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>

#ifdef GRIDFTP_IRODS_MODULE
#include "rodsLog.h"
#include "rsApiHandler.h"
#include "apiHeaderAll.h"
#include "rods.h"
#endif

#define MSG_LEN 1024

void printInfo(char *formatStr, ...) {


#ifdef GRIDFTP_DEBUG

	char msg[MSG_LEN];
	int i;
	va_list ap;
	va_start(ap, formatStr);
	i = vsnprintf(msg, MSG_LEN - 1, formatStr, ap);
	va_end(ap);


#ifdef GRIDFTP_IRODS_MODULE

	rodsLog(LOG_NOTICE, "GRIDFTP_MODULE: %s", msg);
#else
	fprintf(stdout, "%s\n", msg);
#endif
#endif
}

void printErrorMessage(char *formatStr, ...) {

	char msg[MSG_LEN];
	int i;
	va_list ap;
	va_start(ap, formatStr);
	i = vsnprintf(msg, MSG_LEN - 1, formatStr, ap);
	va_end(ap);


#ifdef GRIDFTP_IRODS_MODULE

	rodsLog(LOG_ERROR, " GRIDFTP_MODULE: %s", msg);
#else
	fprintf(stderr, "ERROR: %s\n", msg);
#endif
}

void printErrorObject(globus_object_t *err) {

	printErrorMessage(globus_object_printable_to_string(err));
	//	printErrorMessage(globus_object_printable_to_string(globus_error_get_cause(err)));
	printErrorMessage(globus_error_get_short_desc(err));

	if (globus_error_get_long_desc(err) != NULL) {
		printErrorMessage(globus_error_get_long_desc(err));
	}
}

int checkResult(globus_result_t result) {

	if (result != GLOBUS_SUCCESS) {

		printErrorObject(globus_error_get(result));
		return GLOBUS_FAILURE;
	}
	return GLOBUS_SUCCESS;
}

#ifdef GRIDFTP_IRODS_MODULE

int setUserProxy(rsComm_t *rsComm) {

	if (NULL == getenv("X509_USER_PROXY")) {

		if (MAX_NAME_LEN < strlen(&(rsComm->myEnv.rodsHome)) + strlen(proxy_file)) {

			printInfo("setUserProxy failed: buffer too small for filename");
			return 1;
		}

		dataObjInfo_t *dataObjInfo = NULL;
		dataObjInp_t dataObjInp;
		bzero(&dataObjInp,sizeof(dataObjInp));

		strncpy(dataObjInp.objPath, &(rsComm->myEnv.rodsHome), strlen(&(rsComm->myEnv.rodsHome)));
		strncat(dataObjInp.objPath, proxy_file, strlen(proxy_file));

		if (0 == getDataObjInfo(rsComm, &dataObjInp, &dataObjInfo, NULL, 1)) {

			setenv("X509_USER_PROXY", dataObjInfo->filePath, 1);
			printInfo("X509_USER_PROXY: %s", getenv("X509_USER_PROXY"));
		}
		else {
			printInfo("rodsHome: %s", rsComm->myEnv.rodsHome);
			printInfo("setUserProxy: getDataObjInfo failed for %s", dataObjInp.objPath);
			freeAllDataObjInfo(dataObjInfo);
			return 1;
		}
		freeAllDataObjInfo(dataObjInfo);
	}

	return 0;
}

int msiGridftpTest(ruleExecInfo_t *rei) {
	RE_TEST_MACRO("    msiGridftpTest log");
	return 1;
}
#endif
